    ##### ------------------------------------------------------------------------
    ##### Favorites - Part 1 - Reading and Writing- in CV Class - favorites_reading_and_writing.py
    ##### ------------------------------------------------------------------------

import os
from tkinter import messagebox
import time

from modules.initial_system_scanning_and_data_initialization import read_watcher_output


def read_favorites(app):
    """
    Reads Favorites.txt, checks against WatcherOutput.txt AND zip_structure.txt
    for config existence, and returns a set of valid favorite configurations.
    """
    favorites = set()
    valid_favorites = set()
    watcher_filepaths = set() # Set to store filepaths from WatcherOutput.txt
    zip_structure_lines = set() # Set to store lines from zip_structure.txt

    watcher_output_file = os.path.join(app.script_dir, "WatcherOutput.txt")
    zip_structure_file = os.path.join(app.script_dir, "zip_structure.txt") # Define zip_structure.txt path

    # --- 1. Read Favorites.txt ---
    if os.path.exists(app.favorites_file_path):
        with open(app.favorites_file_path, 'r') as f:
            for line in f:
                line = line.strip()
                if line:
                    favorites.add(line)

    # --- 2. Read WatcherOutput.txt and collect filepaths ---
    if os.path.exists(watcher_output_file):
        check_mods, check_configs, user_vehicles_files, config_pics_custom_files, mods_files, repo_files, vanilla_files, last_zip_count = read_watcher_output(watcher_output_file)
        # Collect all filepaths from WatcherOutput.txt
        watcher_filepaths.update(user_vehicles_files.keys())
        watcher_filepaths.update(config_pics_custom_files.keys())
        watcher_filepaths.update(mods_files.keys())
        watcher_filepaths.update(repo_files.keys())
        watcher_filepaths.update(vanilla_files.keys())

    # --- NEW: 3. Read zip_structure.txt and collect lines ---
    if os.path.exists(zip_structure_file):
        try:
            with open(zip_structure_file, 'r', encoding='utf-8') as f:
                for line in f:
                    line = line.strip()
                    if line:
                        zip_structure_lines.add(line) # Add each line from zip_structure.txt
        except Exception as e:
            print(f"Warning: Error reading zip_structure.txt: {e}")
    # --- NEW: 3. Read zip_structure.txt and collect lines ---


    # --- 4. Check favorites against WatcherOutput and zip_structure ---
    for fav_key in favorites:
        parts = fav_key.split('|')
        if len(parts) == 2:
            folder_name, pc_filename = parts
            pc_filename_base = os.path.splitext(pc_filename)[0] # Get base name without extension
            search_substring = f"{folder_name.lower()}/{pc_filename_base.lower()}" # Construct substring for search

            found_in_watcher_output = False
            for filepath in watcher_filepaths:
                normalized_filepath = filepath.lower().replace('\\', '/')
                if search_substring in normalized_filepath: # Check if substring is IN filepath
                    found_in_watcher_output = True
                    break # No need to check other filepaths if found

            # --- NEW: Check in zip_structure_lines if not found in WatcherOutput ---
            found_in_zip_structure = False
            if not found_in_watcher_output: # Only check zip_structure if NOT found in WatcherOutput
                for zip_line in zip_structure_lines:
                    normalized_zip_line = zip_line.lower().replace('\\', '/')
                    if search_substring in normalized_zip_line: # Check if substring is IN zip_structure line
                        found_in_zip_structure = True
                        break # No need to check other zip_structure lines if found
            # --- NEW: Check in zip_structure_lines if not found in WatcherOutput ---


            if found_in_watcher_output or found_in_zip_structure: # MODIFIED CHECK - OR condition
                valid_favorites.add(fav_key) # Add to valid favorites if found in either file
            else:
                print(f"DEBUG: Favorite '{fav_key}' not found in WatcherOutput.txt OR zip_structure.txt, removing from favorites.") # Debug print for removed favorite
        else:
            print(f"Warning: Invalid favorite key format: '{fav_key}'. Skipping.") # Warning for invalid format

    # --- 5. Update favorites and write back to file ---
    if valid_favorites != favorites: # Only write if there were changes
        app.favorite_configs = valid_favorites
        app.write_favorites()
        print("DEBUG: Favorites.txt updated, invalid entries removed.") # Debug print for update
    else:
        app.favorite_configs = valid_favorites # Ensure app.favorite_configs is updated even if no changes to write
        print("DEBUG: No invalid favorites found, Favorites.txt not updated.") # Debug print for no update

    return app.favorite_configs


def write_favorites(app):
    """Writes the current set of favorite configurations to Favorites.txt."""
    try:
        with open(app.favorites_file_path, "w") as f:
            for fav in sorted(list(app.favorite_configs)):
                f.write(fav + "\n")
        print("Successfully wrote favorites to Favorites.txt") # Confirmation message
    except Exception as e:
        print(f"Error writing to Favorites.txt: {e}") # Error handling message



def is_favorite(app, vehicle_folder, pc_filename):
    """Checks if a configuration is in the favorites list."""
    fav_key = f"{vehicle_folder}|{pc_filename}"
    return fav_key in app.favorite_configs




def add_to_favorites(app, vehicle_folder, pc_filename):
    """Adds a configuration to the favorites list and updates Favorites.txt, showing a scanning window."""
    scanning_win = None  # Initialize scanning_win
    try:
        scanning_win = app.show_scanning_window(text="Added to Favorites")
        fav_key = f"{vehicle_folder}|{pc_filename}"
        app.favorite_configs.add(fav_key)
        app.write_favorites()
    except Exception as e:
        print(f"Error in add_to_favorites: {e}")
        messagebox.showerror("Error", f"Error adding to favorites: {e}", parent=app.master)  # Show error messagebox
    finally:
        time.sleep(0.725)
        if scanning_win:
            scanning_win.destroy()

def remove_from_favorites(app, vehicle_folder, pc_filename):
    """Removes a configuration from the favorites list and updates Favorites.txt, showing a scanning window."""
    scanning_win = None  # Initialize scanning_win
    try:
        scanning_win = app.show_scanning_window(text="Removed from Favorites")
        fav_key = f"{vehicle_folder}|{pc_filename}"
        if fav_key in app.favorite_configs:
            app.favorite_configs.remove(fav_key)
            app.write_favorites()
    except Exception as e:
        print(f"Error in remove_from_favorites: {e}")
        messagebox.showerror("Error", f"Error removing from favorites: {e}", parent=app.master) # Show error messagebox
    finally:
        time.sleep(0.725)
        if scanning_win:
            scanning_win.destroy()



            